{-# OPTIONS -fglasgow-exts #-}
-----------------------------------------------------------------------------------------
{-| Module      :  Process
    Copyright   :  (c) Krasimir Angelov 2003
    License     :  BSD-style

    Maintainer  :  ka2_mail@yahoo.com
    Stability   :  provisional
    Portability :  portable

    The process object is an abstraction which provides access to
    attributes which are global for entire application.
-}
-------------------------------------------------------------------------------

module Graphics.UI.GIO.Process
	( Process, pc
	, start, quit, halt
	, ConfigKey(..), configKey, configKeyDef
	) where

import System.Directory
import Graphics.UI.GIO.Types
import Graphics.UI.GIO.Attributes
import Graphics.UI.GIO.Events
import qualified Graphics.UI.Port as Lib
import Graphics.UI.Port(ConfigKey(..))

data Process = Process

pc :: Process
pc = Process

instance Dismissible Process where
  dismissWidget p = quit
  dismiss         = newEvent (const Lib.getProcessDismissHandler) (const Lib.setProcessDismissHandler) (const Lib.setProcessDismissDefHandler)
  
instance Deadly Process where
  destroyWidget p = halt
  destroy         = newEvent (const Lib.getProcessDestroyHandler) (const Lib.setProcessDestroyHandler) (const Lib.setProcessDestroyDefHandler)

-- | Start the event loop.
start :: String            -- ^ Application name
      -> String            -- ^ Application version
      -> DocumentInterface -- ^ Document interface type for application
      -> [Prop Process]    -- ^ Properties
      -> IO a              -- ^ Startup action
      -> IO ()
start appName appVersion di props io
  = do curdir <- getCurrentDirectory
       Lib.start appName appVersion di ((set pc props >> io) `catch` \err -> quit >> ioError err)
       setCurrentDirectory curdir

-- | Force the event loop to terminate.
quit :: IO Bool
quit = Lib.quit

-- | Force the event loop to terminate.
halt :: IO ()
halt = Lib.halt

configKey :: ConfigKey a => String -> Attr Process a
configKey name = newAttr (\p -> Lib.getConfigKey name) (\p v -> Lib.setConfigKey name v)

configKeyDef :: ConfigKey a => String -> a -> Attr Process a
configKeyDef name defvalue = newAttr (\p -> Lib.getConfigKeyDef name defvalue) (\p v -> Lib.setConfigKey name v)
