-----------------------------------------------------------------------------------------
{-| Module      :  ToolBar
    Copyright   :  (c) Krasimir Angelov 2003
    License     :  BSD-style

    Maintainer  :  ka2_mail@yahoo.com
    Stability   :  provisional
    Portability :  portable

    ToolBar.
-}
-----------------------------------------------------------------------------------------
module Graphics.UI.GIO.ToolBar
		(
		-- * Creation
		  ToolBar,    toolBar
		, ToolButton, toolButtonAt, toolButton
		, ToolCheckButton, toolCheckButtonAt, toolCheckButton
		, ToolRadioButton, toolRadioButtonAt, toolRadioButton, setToolRadioGroup
		, ToolDropDownButton, toolDropDownButtonAt, toolDropDownButton
		, ToolLine,   toolLineAt,   toolLine
             	) where

import Graphics.UI.GIO.Attributes
import Graphics.UI.GIO.Events
import Graphics.UI.GIO.Types
import Graphics.UI.GIO.Menu
import qualified Graphics.UI.Port as Lib

--------------------------------------------------------------------
--  ToolBar
--------------------------------------------------------------------

data ToolBar = ToolBar {htoolbar :: WindowHandle}

toolBar :: String -> PositionType -> Int -> Int -> Int -> [Prop ToolBar] -> IO ToolBar
toolBar name place band_num band_position offset props = do
	bar <- do hwnd <- Lib.createToolBar name place band_num band_position offset
	          return (ToolBar hwnd)
	set bar props
	return bar

instance Deadly ToolBar where
  destroyWidget t = Lib.destroyToolBar (htoolbar t)
  destroy         = newStdEvent htoolbar Lib.getWindowDestroyHandler Lib.setWindowDestroyHandler Lib.setWindowDestroyDefHandler

instance Countable ToolBar where
  count = readAttr "count" (Lib.getToolBarButtonCount . htoolbar)

--------------------------------------------------------------------
--  ToolButton
--------------------------------------------------------------------

data ToolButton = ToolButton {hbutton :: ToolHandle}

toolButtonAt :: Maybe Int -> [Prop ToolButton] -> ToolBar -> IO ToolButton
toolButtonAt pos props (ToolBar hwnd) = do
	btn <- do hbtn <- Lib.insertToolButton hwnd pos
	          return (ToolButton hbtn)
	set btn props
	return btn

toolButton :: [Prop ToolButton] -> ToolBar -> IO ToolButton
toolButton = toolButtonAt Nothing

instance Able ToolButton where
  enabled = newStdAttr hbutton Lib.getToolButtonEnabled Lib.setToolButtonEnabled

instance Commanding ToolButton where
  command =
    newStdEvent hbutton Lib.getToolCommandHandler Lib.setToolCommandHandler Lib.setToolCommandDefHandler

instance HasIcon ToolButton where
  icon  = newStdAttr hbutton Lib.getToolButtonBitmap Lib.setToolButtonBitmap

instance Positioned ToolButton where
  pos = readAttr "pos" (Lib.getToolItemPos . hbutton)

instance ToolTip ToolButton where
  tooltip = newStdAttr hbutton Lib.getToolButtonTip Lib.setToolButtonTip

instance Titled ToolButton where
  title = newStdAttr hbutton Lib.getToolButtonText Lib.setToolButtonText

instance Deadly ToolButton where
  destroyWidget t = Lib.destroyToolItem (hbutton t)
  destroy         = newStdEvent hbutton Lib.getToolDestroyHandler Lib.setToolDestroyHandler Lib.setToolDestroyDefHandler

--------------------------------------------------------------------
--  ToolCheckButton
--------------------------------------------------------------------

data ToolCheckButton = ToolCheckButton {hcheck :: ToolHandle}

toolCheckButtonAt :: Maybe Int -> [Prop ToolCheckButton] -> ToolBar -> IO ToolCheckButton
toolCheckButtonAt pos props (ToolBar hwnd) = do
	btn <- do hbtn <- Lib.insertToolCheckButton hwnd pos
	          return (ToolCheckButton hbtn)
	set btn props
	return btn

toolCheckButton :: [Prop ToolCheckButton] -> ToolBar -> IO ToolCheckButton
toolCheckButton = toolCheckButtonAt Nothing

instance Able ToolCheckButton where
  enabled = newStdAttr hcheck Lib.getToolButtonEnabled Lib.setToolButtonEnabled

instance Commanding ToolCheckButton where
  command = newStdEvent hcheck Lib.getToolCommandHandler Lib.setToolCommandHandler Lib.setToolCommandDefHandler

instance HasIcon ToolCheckButton where
  icon  = newStdAttr hcheck Lib.getToolButtonBitmap Lib.setToolButtonBitmap

instance Positioned ToolCheckButton where
  pos = readAttr "pos" (Lib.getToolItemPos . hcheck)

instance ToolTip ToolCheckButton where
  tooltip = newStdAttr hcheck Lib.getToolButtonTip Lib.setToolButtonTip

instance Titled ToolCheckButton where
  title = newStdAttr hcheck Lib.getToolButtonText Lib.setToolButtonText

instance Checked ToolCheckButton where
  checked = newStdAttr hcheck Lib.getToolButtonChecked Lib.setToolButtonChecked

instance Deadly ToolCheckButton where
  destroyWidget t = Lib.destroyToolItem (hcheck t)
  destroy            = newStdEvent hcheck Lib.getToolDestroyHandler Lib.setToolDestroyHandler Lib.setToolDestroyDefHandler

--------------------------------------------------------------------
--  ToolRadioButton
--------------------------------------------------------------------

data ToolRadioButton = ToolRadioButton {hradio :: ToolHandle}

toolRadioButtonAt :: Maybe Int -> [Prop ToolRadioButton] -> ToolBar -> IO ToolRadioButton
toolRadioButtonAt pos props (ToolBar hwnd) = do
	btn <- do hbtn <- Lib.insertToolRadioButton hwnd pos
	          return (ToolRadioButton hbtn)
	set btn props
	return btn

toolRadioButton :: [Prop ToolRadioButton] -> ToolBar -> IO ToolRadioButton
toolRadioButton = toolRadioButtonAt Nothing

instance Able ToolRadioButton where
  enabled = newStdAttr hradio Lib.getToolButtonEnabled Lib.setToolButtonEnabled

instance Commanding ToolRadioButton where
  command = newStdEvent hradio Lib.getToolCommandHandler Lib.setToolCommandHandler Lib.setToolCommandDefHandler

instance HasIcon ToolRadioButton where
  icon  = newStdAttr hradio Lib.getToolButtonBitmap Lib.setToolButtonBitmap

instance Positioned ToolRadioButton where
  pos = readAttr "pos" (Lib.getToolItemPos . hradio)

instance ToolTip ToolRadioButton where
  tooltip = newStdAttr hradio Lib.getToolButtonTip Lib.setToolButtonTip

instance Titled ToolRadioButton where
  title = newStdAttr hradio Lib.getToolButtonText Lib.setToolButtonText

instance Checked ToolRadioButton where
  checked = newStdAttr hradio Lib.getToolButtonChecked Lib.setToolButtonChecked

instance Deadly ToolRadioButton where
  destroyWidget t = Lib.destroyToolItem (hradio t)
  destroy         = newStdEvent hradio Lib.getToolDestroyHandler Lib.setToolDestroyHandler Lib.setToolDestroyDefHandler

setToolRadioGroup :: [ToolRadioButton] -> IO ()
setToolRadioGroup btns = Lib.setToolRadioGroup (map hradio btns)

--------------------------------------------------------------------
--  ToolDropDownButton
--------------------------------------------------------------------

data ToolDropDownButton = ToolDropDownButton {hdropdown :: ToolHandle}

toolDropDownButtonAt :: Maybe Int -> Menu -> [Prop ToolDropDownButton] -> ToolBar -> IO ToolDropDownButton
toolDropDownButtonAt pos menu props (ToolBar hwnd) = do
	btn <- do hbtn <- Lib.insertToolDropDownButton hwnd (hmenu menu) pos
	          return (ToolDropDownButton hbtn)
	set btn props
	return btn

toolDropDownButton :: Menu -> [Prop ToolDropDownButton] -> ToolBar -> IO ToolDropDownButton
toolDropDownButton = toolDropDownButtonAt Nothing

instance Able ToolDropDownButton where
  enabled = newStdAttr hdropdown Lib.getToolButtonEnabled Lib.setToolButtonEnabled

instance HasIcon ToolDropDownButton where
  icon  = newStdAttr hdropdown Lib.getToolButtonBitmap Lib.setToolButtonBitmap

instance Positioned ToolDropDownButton where
  pos = readAttr "pos" (Lib.getToolItemPos . hdropdown)

instance ToolTip ToolDropDownButton where
  tooltip = newStdAttr hdropdown Lib.getToolButtonTip Lib.setToolButtonTip

instance Titled ToolDropDownButton where
  title = newStdAttr hdropdown Lib.getToolButtonText Lib.setToolButtonText

instance Deadly ToolDropDownButton where
  destroyWidget t = Lib.destroyToolItem (hdropdown t)
  destroy         = newStdEvent hdropdown Lib.getToolDestroyHandler Lib.setToolDestroyHandler Lib.setToolDestroyDefHandler

--------------------------------------------------------------------
--  ToolLine
--------------------------------------------------------------------

data ToolLine = ToolLine {hline :: ToolHandle}

toolLineAt :: Maybe Int -> [Prop ToolLine] -> ToolBar -> IO ToolLine
toolLineAt pos props (ToolBar hwnd) = do
	line <- do hline <- Lib.insertToolLine hwnd pos
	           return (ToolLine hline)
	set line props
	return line

toolLine :: [Prop ToolLine] -> ToolBar -> IO ToolLine
toolLine = toolLineAt Nothing

instance Positioned ToolLine where
  pos = readAttr "pos" (Lib.getToolItemPos . hline)

instance Deadly ToolLine where
  destroyWidget t = Lib.destroyToolItem (hline t)
  destroy         = newStdEvent hline Lib.getToolDestroyHandler Lib.setToolDestroyHandler Lib.setToolDestroyDefHandler
