module Main where

import Graphics.UI.GIO

codecsToFilesFilter :: [Codec] -> [(String,[String])]
codecsToFilesFilter = map codeToFileFilter
  where
    codeToFileFilter :: Codec -> (String,[String])
    codeToFileFilter codec = (codecDescr codec,map (\e->"*."++e) (codecExtensions codec)) 
    
main
  = start "Bitmap Viewer" "1.0" MDI [] $
    do mfile  <- menu [title =: "&File"] mainMenu
       mopen  <- menuitem [title =: "&Open"] mfile
       mclose <- menuitem [title =: "&Close", enabled =: False] mfile
       menuline mfile
       mexit  <- menuitem [title =: "&Exit", on command =: quit >> return ()] mfile
       
       set mopen [on command =: cmdopen mclose]
       return ()
  where
    cmdopen mclose
      = do codes <- getAvailableCodecs
           fnames <- runInputFilesDialog "Open" (codecsToFilesFilter codes) Nothing
           mapM_ (openBitmapWindow mclose) fnames

    cmdclose w mclose
      = do set mclose [enabled =: False, off command]
           dismissWidget w
           return ()

    paintImage image can updFrame updAreas
      = do drawBitmap (pt 0 0) image can
      
    openBitmapWindow mclose fname = do
      bitmap <- readBitmap fname []
      bmpSize <- get bitmap size
      w <- window [ title  =: fname
		  , domain =: bmpSize
		  , view =: sz 300 300
		  , on paint =: paintImage bitmap
		  ]
      set mclose [enabled =: True, on  command =: cmdclose w mclose]
      set w [ on activate   =: set mclose [enabled =: True,  on  command =: cmdclose w mclose]
	    , on deactivate =: set mclose [enabled =: False, off command]
	    ]
      showWindow w
